import java.io.*;
import java.util.*;

public class CSVCreator {
  /**
   * @param args
   */
  List standardAgents;
  List entrantAgents;
  Map startingPoints;
  // String priceFileName;
  int maxDate;
  int numOfSessionsPerDay;
  int numOfExp1 = 50;
  int numOfExp2 = 50;
  int numOfExp3 = 250;
  Random rand = new Random();

  public static final String LOGIN_NAME = "LOGIN_NAME";

  public static final String CLASS_NAME = "CLASS_NAME";

  public static final String SPOT_FILENAME = "SPOT_FILENAME";

  public static final String STARTING_POINT = "STARTING_POINT";

  public CSVCreator() {
    standardAgents = new ArrayList();
    entrantAgents = new ArrayList();
    startingPoints = new HashMap();
  }


  private void createFiles(String dir, List members, String spotFilename, int startingPoint) {
    String resourceDirName = dir + "/resources";
    File resourceDir = new File(resourceDirName);
    resourceDir.mkdirs();
    createMembers(resourceDirName, members);
    createTimeSeriesDefinitions(resourceDirName, spotFilename, startingPoint);
    createSpotFile(resourceDirName, spotFilename);
  }

  private void createMembers(String dir, List members) {
    File outputFile = new File(dir + "/Members.csv");
    try {
      PrintWriter pw = new PrintWriter(new FileWriter(outputFile));
      pw.println("LoginName,Password,Attribute,Connection,Access(Allow_All)(CHART:BOARD:ORDER:ORDER_RESULT:PROFIT:POSITION:STDOUT),RealName,SystemParameters,Seed,InitialCash,TradingUnit,FeePerUnit,MarginRate,MaxLoan,Interest");
      Iterator i = members.iterator();
      int counter = 1;
      while (i.hasNext()) {
        HashMap info = (HashMap)i.next();
        String loginName = (String)info.get(CSVCreator.LOGIN_NAME);
        String className = (String)info.get(CSVCreator.CLASS_NAME);
        String str = loginName + ",passwd" + counter + ",Machine,Local,,"
          + "strategy." + className + ",," + Math.abs(rand.nextInt())
            + ",1000000000,1000,0,300000,30000000,0.1";
        pw.println(str);
        counter++;
      }
      pw.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  private void createTimeSeriesDefinitions(String dir, String spotFilename, int startingPoint) {
    File outputFile = new File(dir + "/TimeSeriesDefinitions.csv");
    try {
      PrintWriter pw = new PrintWriter(new FileWriter(outputFile));
      pw.println("Nickname,Filename,BrandName,StartStep,MaxDate,NoOfSessionsPerDay");
      pw.println("Default,resource:" + spotFilename + ",J30,"
                 + startingPoint + "," + maxDate + ","
                 + numOfSessionsPerDay);
      pw.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  private void createSpotFile(String dir, String filename) {
    File inputFile = new File(filename);
    File outputFile = new File(dir + "/" + filename);
    FileReader fr;
    FileWriter fw;
    String str = null;
    try {
      fr = new FileReader(inputFile);
      BufferedReader br = new BufferedReader(fr);
      fw = new FileWriter(outputFile);
      PrintWriter pw = new PrintWriter(fw);
      while ((str = br.readLine()) != null) {
        pw.println(str);
      }
      br.close();
      pw.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void createAll() {
    createExp1Files();
    createExp2Files();
    createExp3Files();
  }

  private void createExp1Files() {
    Iterator i = entrantAgents.iterator();
    while (i.hasNext()) {
      List list = new ArrayList();
      HashMap info = (HashMap)i.next();
      list.add(info);
      Iterator k = standardAgents.iterator();
      while (k.hasNext()) {
        list.add(k.next());
      }
      // printList(list);
      String name = (String)info.get(CSVCreator.LOGIN_NAME);
      String dir = "exp1_" + name;
      Iterator j = startingPoints.keySet().iterator();
      while (j.hasNext()) {
        String situation = (String)j.next();
        HashMap spotInfo = (HashMap)startingPoints.get(situation);
        String spotFilename = (String)spotInfo.get(SPOT_FILENAME);
        int startingPoint = ((Integer)spotInfo.get(STARTING_POINT)).intValue();
        for (int expNo = 0; expNo < numOfExp1; ++expNo) {
          String dirName = dir + "_" + situation + "_" + expNo;
          File directory = new File(dirName);
          directory.mkdir();
          createFiles(dirName, list, spotFilename, startingPoint);
        }
      }
    }
  }

  private void createExp2Files() {
    List list = new ArrayList();
    Iterator i = entrantAgents.iterator();
    while (i.hasNext()) {
      list.add(i.next());
    }
    i = standardAgents.iterator();
    while (i.hasNext()) {
      list.add(i.next());
    }
    Iterator j = startingPoints.keySet().iterator();
    while (j.hasNext()) {
      String situation = (String) j.next();
      HashMap spotInfo = (HashMap)startingPoints.get(situation);
      String spotFilename = (String)spotInfo.get(SPOT_FILENAME);
      int startingPoint = ((Integer)spotInfo.get(STARTING_POINT)).intValue();
      for (int expNo = 0; expNo < numOfExp2; ++expNo) {
        String dirName = "exp2_" + situation + "_" + expNo;
        createFiles(dirName, list, spotFilename, startingPoint);
      }
    }
  }

  private void createExp3Files() {
    for (int i = 1; i <= numOfExp3; i++) {
      List list = new ArrayList();
      Iterator itr = entrantAgents.iterator();
      while (itr.hasNext()) {
        list.add(itr.next());
      }
      itr = standardAgents.iterator();
      while (itr.hasNext()) {
        list.add(itr.next());
      }
      dropHalf(list);
      Iterator j = startingPoints.keySet().iterator();
      while (j.hasNext()) {
        String situation = (String) j.next();
        String dirName = "exp3_" + i + "_" + situation;
        HashMap spotInfo = (HashMap)startingPoints.get(situation);
        String spotFilename = (String)spotInfo.get(SPOT_FILENAME);
        int startingPoint = ((Integer)spotInfo.get(STARTING_POINT)).intValue();
        createFiles(dirName, list, spotFilename, startingPoint);
      }
    }
  }

  private void dropHalf(List list) {
    int halfSize = list.size() / 2;
    for (int i = 0; i < halfSize; i++) {
      list.remove(rand.nextInt(list.size()));
    }
  }

  private void initialize(String stdAgentFile, String entrantAgentFile, String priceDefinitionFile,
                          int noOfExp1, int noOfExp2, int noOfExp3) {
    setAgents(stdAgentFile, standardAgents);
    setAgents(entrantAgentFile, entrantAgents);
    setStartingPoints(priceDefinitionFile);
    this.numOfExp1 = noOfExp1;
    this.numOfExp2 = noOfExp2;
    this.numOfExp3 = noOfExp3;
  }

  private void setStartingPoints(String filename) {
    File file = new File(filename);
    FileReader fr = null;
    String str = null;
    try {
      fr = new FileReader(file);
      BufferedReader br = new BufferedReader(fr);
      while ((str = br.readLine()) != null) {
        StringTokenizer st = new StringTokenizer(str);
        if (str.startsWith("#")) {
          continue;
        } else if (str.startsWith("maxDate")) {
          st.nextToken();
          maxDate = Integer.parseInt(st.nextToken());
          continue;
        } else if (str.startsWith("numOfSessionsPerDay")) {
          st.nextToken();
          numOfSessionsPerDay = Integer.parseInt(st.nextToken());
          continue;
        }
        String situation = st.nextToken();
        String spotFilename = st.nextToken();
        int startingPoint = Integer.parseInt(st.nextToken());
        HashMap info = new HashMap();
        info.put(CSVCreator.SPOT_FILENAME, spotFilename);
        info.put(CSVCreator.STARTING_POINT, new Integer(startingPoint));
        startingPoints.put(situation, info);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void setAgents(String filename, List agentList) {
    File file = new File(filename);
    FileReader fr = null;
    String str = null;
    try {
      fr = new FileReader(file);
      BufferedReader br = new BufferedReader(fr);
      while ((str = br.readLine()) != null) {
        if (str.startsWith("#")) {
          continue;
        }
        StringTokenizer st = new StringTokenizer(str);
        HashMap info = new HashMap();
        info.put(CSVCreator.LOGIN_NAME, st.nextToken());
        info.put(CSVCreator.CLASS_NAME, st.nextToken());
        agentList.add(info);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void printList(List list) {
    Iterator i = list.iterator();
    while (i.hasNext()) {
      System.out.println(i.next());
    }
  }

  private void printMap(Map map) {
    Iterator i = startingPoints.keySet().iterator();
    while (i.hasNext()) {
      Object obj = i.next();
      System.out.println(obj + ", " + startingPoints.get(obj));
    }
  }

  private void PrintAll() {
    System.out.println("************Parameters***************");
    // System.out.println("priceFileName = " + priceFileName);
    System.out.println("maxDate = " + maxDate);
    System.out.println("numOfSessionsPerDay = " + numOfSessionsPerDay);
    System.out.println("************Standards***************");
    printList(standardAgents);
    System.out.println("************Entrants***************");
    printList(entrantAgents);
    System.out.println("************StartingPoints***************");
    printMap(startingPoints);
  }

  public static void main(String[] args) {
    if (args.length != 6) {
      System.out.println("Usage: java CSVCreator <StandardsFile> <EntrantsFile> <PriceFile>"
                         + " <noOfExp1> <noOfExp2> <noOfExp3>");
      System.exit(-1);
    }
    String stdAgentFile = args[0];
    String entrantAgentFile = args[1];
    String priceDefinitionFile = args[2];
    int noOfExp1 = Integer.parseInt(args[3]);
    int noOfExp2 = Integer.parseInt(args[4]);
    int noOfExp3 = Integer.parseInt(args[5]);
    CSVCreator creator = new CSVCreator();
    creator.initialize(stdAgentFile, entrantAgentFile, priceDefinitionFile,
                       noOfExp1, noOfExp2, noOfExp3);
    creator.createAll();
  }
  
}
