/*
 Copyright (c) 2000 Rikiya Fukumoto
 Copyright (c) 2002 U-Mart Project
 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
 "Software"), to deal in the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ---------------------------------------------------------------------
eNjJ@̈łMACDpĔsG[WFg
qyBuyMACD.java
MovingAverageStrategy.java
QlɂčB
 */

package strategy;

import java.util.*;

public class T05_MACDStrategy extends Strategy {

  /** ʂ̍ől̃ftHgl */
  public static final int DEFAULT_MAX_QUANT = 50;

  /** ʂ̍ŏl̃ftHgl */
  public static final int DEFAULT_MIN_QUANT = 15;

  /** |WV̍ől̃ftHgl */
  public static final int DEFAULT_MAX_POSITION = 30;

  /** 瓾SẲi񂪖̂ƂɎgĩftHgl */
  public static final int DEFAULT_NOMINAL_PRICE = 3000;

  /** ZړςvZԂ̃ftHgl */
  public static final int DEFAULT_SHORT_REFERENCE_TERM = 6;

  /** ړςvZԂ̃ftHgl */
  public static final int DEFAULT_LONG_REFERENCE_TERM = 13;

  /** VOivZԂ̃ftHgl */
  public static final int DEFAULT_SIGNAL_REFERENCE_TERM = 4;

  /**  */
  private Random fRandom;

  /** ʂ̍ől */
  private int fMaxQuant = DEFAULT_MAX_QUANT;

  /** ʂ̍ŏl */
  private int fMinQuant = DEFAULT_MIN_QUANT;

  /** |WV̍ől */
  private int fMaxPosition = DEFAULT_MAX_POSITION;

  /** 瓾SẲi񂪌̂ƂɎgi */
  private int fNominalPrice = DEFAULT_NOMINAL_PRICE;

  /** ZړςvZԂ̃ftHgl */
  private int fShortReferenceTerm = DEFAULT_SHORT_REFERENCE_TERM;

  /** ZړςvZԂ̃ftHgl */
  private int fLongReferenceTerm = DEFAULT_LONG_REFERENCE_TERM;

  /** ZړςvZԂ̃ftHgl */
  private int fSignalReferenceTerm = DEFAULT_SIGNAL_REFERENCE_TERM;

  /* Constructor for Agent Initialization */
  public T05_MACDStrategy(int seed) {
    /* If you need some initialization of e.g., instance variables
       write here.
     */
    fRandom = new Random(seed); // Initialization of random sequence.
  }

  /* Method of making order. If you submit a single order in one period,
     keep it as it is. If you want to submit several orders in one period,
     prepare methods other than getOrder and repeat orderRequest.
   */
  public void action(int[] spotPrices, int[] futurePrices,
                     int pos, long money, int restDay) {
    /* Class order has instance variables as follows
       order.price: limit price,
       order.quant: quantity,
       order.buysell: no order(0), sell(1) or buy(2)
     */
    /* decide 'order' (here we use method getOrder described below),
       and call method orderRequest to submit order.
       If you want to submit several orders, repeat such procedure
     */
    Order order = getOrder(spotPrices, futurePrices, pos, money, restDay);
    orderRequest(order);
  }

  /* Method of making single order. Implement your trading strategy rewriting
     This method */
  public Order getOrder(int[] spotPrices, int[] futurePrices,
                        int pos, long money, int restDay) {
    /* Information available for trading:
       spotPrices[]: Time series of spot prices. It provides 120 elements from spotPrices[0]
       to spotPrices[119].  The element spotPrices[119] is the latest.
       futurePrices[]: Time series of futures price. It provides 60 elements from futurePrices[0]
       to futurePrices[59]. The element futurePrices[59] is the latest.
       Before opening the market, same values with spot prices are given.
       If no contract is made in the market, value -1 is given.
         pos: Position of the agent. Positive is buying position. Negative is selling.
       money: Available cash. Note that type 'long' is used because of needed precision.
       restDay: Number of remaining transaction to the closing of the market.
     */
    Order order = new Order(); // Object to return values.

    /* VOivZ邽߂ fSignalReferenceTerm+1MACDvZ*/

	/*MACDi[邽߂̔zB[0]ŐV̒lB*/
	double [] MACD;
    MACD = new double [fSignalReferenceTerm+1];

	for(int i = 0; i <= fSignalReferenceTerm;i++){
		
	    double shortTermEMA = calculateEMA(spotPrices,fShortReferenceTerm, i);

    	double longTermEMA = calculateEMA(spotPrices,fLongReferenceTerm, i);

		MACD[i] = shortTermEMA - longTermEMA;
	}

    double signal = calculateSignal(MACD,fSignalReferenceTerm, 0);
    double previousSignal = calculateSignal(MACD,fSignalReferenceTerm, 1);

	//MACDVOiɔƂ
	//MACDVOiɔƂ
    // Decide BUY or SELL
    if (MACD[0] == MACD[1]) {
      order.buysell = Order.NONE;
      return order;
    } else if (MACD[1] < MACD[0]) {
      if (previousSignal > MACD[1]
          && signal > MACD[0]) {
        order.buysell = Order.NONE;
        return order;
      } else if (previousSignal < MACD[1] &&
                 signal < MACD[0]) {
        order.buysell = Order.NONE;
        return order;
      } else {
        order.buysell = Order.BUY;
      }
    } else if (MACD[1] > MACD[0]) {
      if (previousSignal > MACD[1]
          && signal > MACD[0]) {
        order.buysell = Order.NONE;
        return order;
      } else if (previousSignal < MACD[1] &&
                 signal < MACD[0]) {
        order.buysell = Order.NONE;
        return order;
      } else {
        order.buysell = Order.SELL;
      }
    }
    // Cancel decision if it may increase absolute value of the position
    if (order.buysell == Order.BUY) {
      if (pos > fMaxPosition) {
        order.buysell = Order.NONE;
        return order;
      }
    } else if (order.buysell == Order.SELL) {
      if (pos < -fMaxPosition) {
        order.buysell = Order.NONE;
        return order;
      }
    }
    /* Decide limit price */
    int latestPrice = getLatestPrice(spotPrices);
    boolean flag = false;
    int previousLatestPrice = -1;
    for (int i = spotPrices.length - 1; i >= 0; --i) {
      if (spotPrices[i] >= 0 && flag == false) {
        flag = true;
      } else if (spotPrices[i] >= 0 && flag == true) {
        previousLatestPrice = spotPrices[i];
        break;
      }
    }
    int widthOfPrice = Math.abs(latestPrice - previousLatestPrice);
    while (true) {
      if (order.buysell == Order.BUY) {
        order.price = latestPrice + widthOfPrice +
            (int) (widthOfPrice / 4 * fRandom.nextGaussian());
      } else if (order.buysell == Order.SELL) {
        order.price = latestPrice - widthOfPrice +
            (int) (widthOfPrice / 4 * fRandom.nextGaussian());
      }
      if (order.price > 0) {
        break;
      }
    } // Add normal random number around prevPrice
    // with the standard deviation a quarter of widthOfPrice.
    // If it gets negative, repeat again.
    // Decide quantity of order between [minQuant, maxQuant] randomly.
    order.quant = fMinQuant + fRandom.nextInt(fMaxQuant - fMinQuant + 1);


    // Message
    message(Order.buySellToString(order.buysell)
            + ", price = " + order.price
            + ", volume = " + order.quant
            + ", MACD = " + MACD[0]
            + ", signal = " + signal
            + ", previous MACD = " + MACD[1]
            + ", previous signal = " + previousSignal);
    return order;
  }

  // EMAvZ 
  private double calculateEMA(int[] prices, int referenceTerm, int startOfIndex) {
    double EMA = 0.0; //
    int count = 0; // consider only valid prices

    for (int i = 0; i < referenceTerm; ++i) {
      // skip if the price is negative that means no contract
      if (prices[prices.length - 1 - startOfIndex - i] > 0) {
			EMA = (1.0 - (2.0 / (i+1))) * EMA + (2.0 / (i+1)) * prices[prices.length - 1 - startOfIndex - i];
        count++;
      }
    }
    if (count == 0) { // Make no order if all the prices are invalid
      return -1.0;
    }
    return EMA;
  }

  //VOivZ
  private double calculateSignal(double[] prices, int referenceTerm, int startOfIndex) {
    double EMA = 0.0; 
    int count = 0; // consider only valid prices
    // Sum up futures prices
    for (int i = 0; i < referenceTerm; ++i) {
			EMA = (1.0 - (2.0 / (i+1))) * EMA + (2.0 / (i+1)) * prices[prices.length - 1 - startOfIndex - i];
    }
    return EMA;
  }

  /* (non-Javadoc)
   * @see strategy.UBaseStrategy#setParameters(java.lang.String[])
   */
  public void setParameters(String[] args) {
    super.setParameters(args);
    for (int i = 0; i < args.length; ++i) {
      StringTokenizer st = new StringTokenizer(args[i], "= ");
      String key = st.nextToken();
      String value = st.nextToken();
      if (key.equals("MinQuant")) {
        fMinQuant = Integer.parseInt(value);
      } else if (key.equals("MaxQuant")) {
        fMaxQuant = Integer.parseInt(value);
      } else if (key.equals("MaxPosition")) {
        fMaxPosition = Integer.parseInt(value);
      } else if (key.equals("ShortReferenceTerm")) {
        fShortReferenceTerm = Integer.parseInt(value);
      } else if (key.equals("LongReferenceTerm")) {
        fLongReferenceTerm = Integer.parseInt(value);
      } else if (key.equals("SignalReferenceTerm")) {
		fSignalReferenceTerm = Integer.parseInt(value);
	  } else {
        message("Unknown parameter:" + key + " in MACDStrategy.setParameters");
      }
    }
  }

}

