/*

Copyright (c) 1999 Hiroshi Sato
Copyright (c) 2000 Rikiya Fukumoto
Copyright (c) 2002 U-Mart Project

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
---------------------------------------------------------------------

TrendStrategy.java             2002/01/31

A class describing an agent's strategy, which buys or sells following
the market trend captured by difference between the last two futures price.
The limit price is set randomly around the latest futures price, and
quantity of the order is set randomly within a prescribed range.
Position of the agent is also considered in decision making.

 */

package strategy;

import java.util.*;
import strategy.*;

public class A0027279aStrategy extends Strategy {

	/** ĩ̕ftHgl */
	public static final int DEFAULT_WIDTH_OF_PRICE = 20;

	/** ʂ̍ől̃ftHgl */
	public static final int DEFAULT_MAX_QUANT = 100;

	/** ʂ̍ŏl̃ftHgl */
	public static final int DEFAULT_MIN_QUANT = 100;

	/** |WV̍ől̃ftHgl */
	public static final int DEFAULT_MAX_POSITION = 1000;

	/** 瓾SẲi񂪖̂ƂɎgĩftHgl */
	public static final int DEFAULT_NOMINAL_PRICE = 3000;

	/**  */
	private Random fRandom;

	/** i̕ */
	private int fWidthOfPrice = DEFAULT_WIDTH_OF_PRICE;

	/** ʂ̍ől */
	private int fMaxQuant = DEFAULT_MAX_QUANT;

	/** ʂ̍ŏl */
	private int fMinQuant = DEFAULT_MIN_QUANT;

	/** |WV̍ől */
	private int fMaxPosition = DEFAULT_MAX_POSITION;

	/** 瓾SẲi񂪌̂ƂɎgi */
	private int fNominalPrice = DEFAULT_NOMINAL_PRICE;


	public A0027279aStrategy(int seed) {
		fRandom = new Random(seed);
	}

	public void action(int[] spotPrices, int[] futurePrices,
		                      int pos, long money, int restDay) {
		/* Class order has instance variables as follows
		   order.price: limit price,
		   order.quant: quantity,
		   order.buysell: no order(0), sell(1) or buy(2)
		    */
		/* decide 'order' (here we use method getOrder described below),
		   and call method orderRequest to submit order.
		   If you want to submit several orders, repeat such procedure
		*/
		Order order = getOrder(spotPrices, futurePrices, pos, money, restDay);
		orderRequest(order);
	}

	public Order getOrder(int[] spotPrices, int[] futurePrices,
		                          int pos, long money, int restDay) {
		Order order = new Order();

		int i = 0;
		int j = 0;
		int totalPrice = 0;
		for(i = 0;i <= futurePrices.length - 1;i++){
			if(futurePrices[i] == -1){
				j++;
			}
			else{
				totalPrice += futurePrices[i];
			}
		}
		int averagePrice = Math.round(totalPrice / (futurePrices.length - j));  // ߋ̐敨i̕ϒl
		int latestFuturePrice = futurePrices[futurePrices.length - 1];  // ŐV̐敨i
		int latestSpotPrice = spotPrices[spotPrices.length - 1];  // ŐV̌i
		int[] beforeFuturePrice = {futurePrices[futurePrices.length - 9], futurePrices[futurePrices.length - 17]};  // 1,2O̐敨i
		double[] diff = {latestFuturePrice - beforeFuturePrice[0], latestFuturePrice - beforeFuturePrice[1]};  // 1,2O̐敨iƍŐV敨i̍
		double point = (latestFuturePrice - latestSpotPrice) * 5 + (latestFuturePrice - averagePrice) * 2 - (diff[0] + diff[1]) / 2;  // f̎wWƂȂ|CgZo

		System.out.println("A0027279aStrategy: point = " + point);

		if (latestFuturePrice > 0 && beforeFuturePrice[0] > 0 && beforeFuturePrice[1] > 0) {
			if (point >= 100) {  // 100|Cgȏ͔̎
				if (diff[0] > 0 && diff[1] > 0){
					order.buysell = Order.NONE;  // Ai㏸X͔Ȃ
					return order;
				}
				order.buysell = Order.SELL;
			} else if (point <= -100) {  // -100|Cgȉ͔̎
				if (diff[0] < 0 && diff[1] < 0){
					order.buysell = Order.NONE;  // Ai~X͔Ȃ
					return order;
				}
				order.buysell = Order.BUY;
			} else {
				order.buysell = Order.NONE;  // -100 < point < 100 ̂Ƃ͉Ȃ
				return order;
			}
		} else {
			order.buysell = Order.NONE;
			return order;
			// if price is not well defined.
		}
		// Cancel decision if it may increase absolute value of the position
		if (order.buysell == Order.BUY) {
			if (pos > fMaxPosition) {
				order.buysell = Order.NONE;
				return order;
			}
		} else if (order.buysell == Order.SELL) {
			if (pos < -fMaxPosition) {
				order.buysell = Order.NONE;
				return order;
			}
		}
		int prevPrice = getLatestPrice(futurePrices);
		if (prevPrice == -1)
			prevPrice = getLatestPrice(spotPrices); // use spot price instead
		if (prevPrice == -1)
			prevPrice = fNominalPrice; // use nominal value instead
		while (true) {
			order.price = prevPrice + (int) (fWidthOfPrice * fRandom.nextGaussian());
			if (order.price > 0)
				break;
		}
		order.quant = fMinQuant + fRandom.nextInt(fMaxQuant - fMinQuant + 1);
		return order;
	}

	/* (non-Javadoc)
	 * @see strategy.UBaseStrategy#setParameters(java.lang.String[])
	 */
	public void setParameters(String[] args) {
		super.setParameters(args);
		for (int i = 0; i < args.length; ++i) {
			StringTokenizer st = new StringTokenizer(args[i], "= ");
			String key = st.nextToken();
			String value = st.nextToken();
			// System.err.println(key + "=" + value);
			if (key.equals("WidthOfPrice")) {
				fWidthOfPrice = Integer.parseInt(value);
			} else if (key.equals("MinQuant")) {
				fMinQuant = Integer.parseInt(value);
			} else if (key.equals("MaxQuant")) {
				fMaxQuant = Integer.parseInt(value);
			} else if (key.equals("MaxPosition")) {
				fMaxPosition = Integer.parseInt(value);
			} else {
				System.err.println("Unknown parameter:" + key
									   + " in A0027279aStrategy.setParameters");
				System.exit(5);
			}
		}
	}

}


