/*
A class describing an agent's strategy, which simultaneously submits a buying
order with a limit price lower than and a selling order higher than the
previous price by a prescribed spread.
The quantity of the order is set randomly within a prescribed range.
Position of the agent is also considered in decision making.
 */

package strategy;
import java.util.*;
public class Psycho4Strategy extends Strategy {

    //************ Ǝ̐ݒ荀 ******************************

    /** TCRWJC̏ݒij*/
    private int BUY_PSYCHO_VALUE = 4;

    /** TCRWJC̏ݒij*/
    private int SELL_PSYCHO_VALUE =4;
    //************************************************************************

    /** Default value of Order Price Width */
    public static final int DEFAULT_WIDTH_OF_PRICE = 20;

    /** Default value of Maximum Order Volume */
    public static final int DEFAULT_MAX_QUANT = 50;

    /** Default value of Minimum Order Volume */
    public static final int DEFAULT_MIN_QUANT = 10;

    /** Default value of Maximum Position */
    public static final int DEFAULT_MAX_POSITION = 300;

    /** Default value of Spread Ration */
    public static final double DEFAULT_SPREAD_RATIO = 0.01;

    /** Random Number Generator */
    private Random fRandom;

    /** Width of Order Price */
    private int fWidthOfPrice = DEFAULT_WIDTH_OF_PRICE;

    /** Maximum Value of Order Volume */
    private int fMaxQuant = DEFAULT_MAX_QUANT;

    /** Minimum Value of Order Volume */
    private int fMinQuant = DEFAULT_MIN_QUANT;

    /** Maximum Position */
    private int fMaxPosition = DEFAULT_MAX_POSITION;

    /** Spread Ratio */
    private double fSpreadRatio = DEFAULT_SPREAD_RATIO;

    /* Constructor for Agent Initialization */

    public Psycho4Strategy(int seed) {
	/* If you need some initialization of e.g., instance variables
	   write here.
	*/
	fRandom = new Random(seed); // Initialization of random sequence.
    }

    /* Method of making order. If you submit a single order in one period,
       keep it as it is. If you want to submit several orders in one period,
       prepare methods other than getOrder and repeat orderRequest.
    */

    /* Method of making single order. Implement your trading strategy rewriting
       This method */

    public Order getOrder(int[] spotPrices, int[] futurePrices,
			  int pos, long money, int restDay) {

	/* Information available for trading:
	   spotPrices[]: Time series of spot prices. It provides 120 elements from spotPrices[0]
	   to spotPrices[119].  The element spotPrices[119] is the latest.
	   futurePrices[]: Time series of futures price. It provides 60 elements from futurePrices[0]
	   to futurePrices[59]. The element futurePrices[59] is the latest.
	   Before opening the market, same values with spot prices are given.
	   If no contract is made in the market, value -1 is given.
	   pos: Position of the agent. Positive is buying position. Negative is selling.
	   money: Available cash. Note that type 'long' is used because of needed precision.
	   restDay: Number of remaining transaction to the closing of the market.  */

	/* CX^Xϐ̐錾I*/
	Order order       = new Order(); // Object to return values.
	PsychologicalLine_buy psychobuy  = new PsychologicalLine_buy(BUY_PSYCHO_VALUE);
	PsychologicalLine_sell psychosell = new PsychologicalLine_sell(SELL_PSYCHO_VALUE);

	int latestFuturePrice = futurePrices[futurePrices.length - 1];
	int latestSpotPrice   = spotPrices[spotPrices.length - 1];

	/*
	  oǂ̌ieeNjJ@VOioǂŔf
	  **.make_signal(spotPrices,futurePrices)== true ȂAVOioĂƂƁB
	  falseȂAVOiƂƂɂȂBƔ͕ʂ̃CX^XɂȂB
	  ̏ꍇAqrhAړσNX̗VOioΒoB
	*/
	if(psychobuy.make_signal(spotPrices, futurePrices) == true && latestFuturePrice < latestSpotPrice){
          order.buysell = Order.BUY;
          order.price = (int) ((double) latestFuturePrice * (1.0 + fSpreadRatio));
        }else if(psychosell.make_signal(spotPrices,futurePrices) == true && latestFuturePrice > latestSpotPrice){
          order.buysell = Order.SELL;
          order.price = (int) ((double) latestFuturePrice * (1.0 - fSpreadRatio));
        }else{
          order.buysell = Order.NONE;
        }


	if (latestFuturePrice < 0) {
	    order.buysell = Order.NONE;
	    return order;
	} // If previous price is invalid, it makes no order.



	if (order.price < 1)
	    order.price = 1;

	/*ʂ̌(Lbĝ܂)*/
	order.quant = fMinQuant + fRandom.nextInt(fMaxQuant - fMinQuant + 1);

	// Message
	message("MyDayTradeStrategy: Buy" + ", price = " + order.price
		+ ", volume = " + order.quant + "  )");

	return order;
    }


    /* (non-Javadoc)
     * @see strategy.UBaseStrategy#setParameters(java.lang.String[])
     *@KvȂI
     */
    public void setParameters(String[] args) {
	super.setParameters(args);
	for (int i = 0; i < args.length; ++i) {
	    StringTokenizer st = new StringTokenizer(args[i], "= ");
	    String key = st.nextToken();
	    String value = st.nextToken();
	    if (key.equals("WidthOfPrice")) {
		fWidthOfPrice = Integer.parseInt(value);
	    } else if (key.equals("MinQuant")) {
		fMinQuant = Integer.parseInt(value);
	    } else if (key.equals("MaxQuant")) {
		fMaxQuant = Integer.parseInt(value);
	    } else if (key.equals("MaxPosition")) {
		fMaxPosition = Integer.parseInt(value);
	    } else if (key.equals("SpreadRatio")) {
		fSpreadRatio = Double.parseDouble(value);
	    } else {
		message("Unknown parameter:" + key
			+ " in RandomStrategy.setParameters");
	    }
	}
    }
}




abstract class UGetSignal{

    /*i*/
    ArrayList spot   = new ArrayList();

    /*敨i*/
    ArrayList future = new ArrayList();

    /*(萔) i̔z̗vf*/
    public static int SPOT_ELEMS   = 120;

    /*(萔j敨i̔z̗vf*/
    public static int FUTURE_ELEMS = 60;

    boolean make_signal(int[] spotPrices, int[] futurePrices){

	/*Ԃl*/
	boolean signal = false;
	
	/*
	  ԂlɂẮA
	  false .... m[VOi
	  true  .... VOiioj
	  ̂QƂȂB
	*/

	/*izXg֕ϊ*/
	for(int i = 0; i < spotPrices.length;i++) spot.add(Integer.toString(spotPrices[i]));

	/* 敨izXg֕ϊ*/
	for(int i = 0; i < futurePrices.length;i++) future.add(Integer.toString(futurePrices[i]));

	/*timing\bh̕Ԓl^ȂVOioiojB*/
	if(timing()){
          signal = true;
        }

	return signal;
    };

 abstract boolean timing();
}

class PsychologicalLine_buy extends UGetSignal{
  int standard;

  public PsychologicalLine_buy(int a){
    standard = a;
  }

  boolean timing(){
    int i;
    int j;
    ArrayList result = new ArrayList();
    double percentage ;
    double total = 0;
    String tuika;
//IlOォȂ1AȂ0Xgɒǉ
    for(i = SPOT_ELEMS-1;i > SPOT_ELEMS - standard -1;i--){
     if(Double.parseDouble((String)spot.get(i)) == -1){
       return false;
     }
        if(Double.parseDouble((String)spot.get(i)) >= Double.parseDouble((String)spot.get(i-1))){
          tuika = "1";
          result.add(tuika);
        }else{
          tuika = "0";
          result.add(tuika);
        }

    }

    for(i= 0;i<standard;i++){
          total = total + Double.parseDouble((String)result.get(i));
        }
        percentage = total/standard*100;
//́25ȉȂAtrue̒lԂ
	    return (percentage <= 25) ? true : false ;
  }
}

class PsychologicalLine_sell extends UGetSignal{
  int standard;

  public PsychologicalLine_sell(int a){
    standard = a;
  }

  boolean timing(){
    int i;
    int j;
    ArrayList result = new ArrayList();
    double percentage ;
    double total = 0;
    String tuika;
//IlOォȂ1AȂ0Xgɒǉ
    for(i = SPOT_ELEMS-1;i > SPOT_ELEMS - standard -1;i--){
     if(Double.parseDouble((String)spot.get(i-1)) == -1){
       return false;
     }

        if(Double.parseDouble((String)spot.get(i)) >= Double.parseDouble((String)spot.get(i-1))){
          tuika = "1";
          result.add(tuika);
        }else{
          tuika = "0";
          result.add(tuika);
        }
    }

    for(i= 0;i<standard;i++){
          total = total + Double.parseDouble((String)result.get(i));
        }
        percentage = total/standard*100;
//́75ȏȂA̎true̒lԂ
	    return (percentage >= 75) ? true : false ;
  }
}

