

/*
A class describing an agent's strategy, which simultaneously submits a buying
order with a limit price lower than and a selling order higher than the
previous price by a prescribed spread.
The quantity of the order is set randomly within a prescribed range.
Position of the agent is also considered in decision making.
 */

package strategy;
import java.util.*;
public class SecondLogisticStrategy extends Strategy {

    //************ ݒ荀 *************************************************
    /** ߎ̍ۂɎQƂ敨̊ */
    private int TERM  = 10;

    //************************************************************************

    /** Default value of Order Price Width */
    public static final int DEFAULT_WIDTH_OF_PRICE = 20;

    /** Default value of Maximum Order Volume */
    public static final int DEFAULT_MAX_QUANT = 50;

    /** Default value of Minimum Order Volume */
    public static final int DEFAULT_MIN_QUANT = 10;

    /** Default value of Maximum Position */
    public static final int DEFAULT_MAX_POSITION = 300;

    /** Default value of Spread Ration */
    public static final double DEFAULT_SPREAD_RATIO = 0.01;

    /** Random Number Generator */
    private Random fRandom;

    /** Width of Order Price */
    private int fWidthOfPrice = DEFAULT_WIDTH_OF_PRICE;

    /** Maximum Value of Order Volume */
    private int fMaxQuant = DEFAULT_MAX_QUANT;

    /** Minimum Value of Order Volume */
    private int fMinQuant = DEFAULT_MIN_QUANT;

    /** Maximum Position */
    private int fMaxPosition = DEFAULT_MAX_POSITION;

    /** Spread Ratio */
    private double fSpreadRatio = DEFAULT_SPREAD_RATIO;

    /** Divergence flag */
    private boolean DivergenceFlag = false;

    /* Constructor for Agent Initialization */

    public SecondLogisticStrategy(int seed) {
	/* If you need some initialization of e.g., instance variables
	   write here.
	*/
	fRandom = new Random(seed); // Initialization of random sequence.
    }

    /* Method of making order. If you submit a single order in one period,
       keep it as it is. If you want to submit several orders in one period,
       prepare methods other than getOrder and repeat orderRequest.
    */

    /* Method of making single order. Implement your trading strategy rewriting
       This method */

    public Order getOrder(int[] spotPrices, int[] futurePrices,
			  int pos, long money, int restDay) {

	/* Information available for trading:
	   spotPrices[]: Time series of spot prices. It provides 120 elements from spotPrices[0]
	   to spotPrices[119].  The element spotPrices[119] is the latest.
	   futurePrices[]: Time series of futures price. It provides 60 elements from futurePrices[0]
	   to futurePrices[59]. The element futurePrices[59] is the latest.
	   Before opening the market, same values with spot prices are given.
	   If no contract is made in the market, value -1 is given.
	   pos: Position of the agent. Positive is buying position. Negative is selling.
	   money: Available cash. Note that type 'long' is used because of needed precision.
	   restDay: Number of remaining transaction to the closing of the market.  */


	/* CX^Xϐ̐錾I*/
	Order order       = new Order(); // Object to return values.

	/*S΍BʂȏȂA߂*/
	if (pos > fMaxPosition) {
	    order.buysell = Order.NONE;
	    return order;
	}

	int latestFuturePrice = futurePrices[futurePrices.length - 1];
	int prevPrice = getLatestPrice(futurePrices);
	int latestSpotPrice   = spotPrices[spotPrices.length - 1];

	if (latestFuturePrice < 0) {
	    order.buysell = Order.NONE;
	    return order;
	} // If previous price is invalid, it makes no order.


	/*敨ƌ̘ƁÃ݂|WVl*/
	if (pos > 20 && (latestFuturePrice - latestSpotPrice) > 50) {
	    order.buysell = Order.SELL;
	    order.price   = prevPrice + (int) (fWidthOfPrice * fRandom.nextGaussian());
	    order.quant   = pos + fMinQuant + fRandom.nextInt(fMaxQuant - fMinQuant + 1);
	    message("Kairi:i:"+futurePrices[59]+" c:"+restDay);
	    return order;
	}else if(pos < -20 && (latestFuturePrice - latestSpotPrice) < -50) {
	    order.buysell = Order.BUY;
	    order.price   = prevPrice + (int) (fWidthOfPrice * fRandom.nextGaussian());
	    order.quant   = (-1 * pos) + fMinQuant + fRandom.nextInt(fMaxQuant - fMinQuant + 1);
	    message("Kairi:i:"+futurePrices[59]+" c:"+restDay);
	    return order;
	}

	/*WXeBXeBbNȐւ̋ߎZo*/
	/*ߋPUɒ*/
	int latestFutureElem = futurePrices.length - 1;
	int TERM = 10;
	double x=0,y=0,x2=0,xy=0;
	double sum_x = 0,sum_y = 0,sum_x2 = 0,sum_xy =0,sum_app = 0;
	int i;
	for(i = latestFutureElem - TERM + 1; i <= latestFutureElem  ; i++){
	    //  if (i == (latestFutureElem - TERM)) dx.add(Integer.parseInt(futurePrices[latestFutureElem - TERM]));
	   x = (futurePrices[i] + futurePrices[i-1])/2;
	   y = (futurePrices[i] - futurePrices[i-1])/x;
	   x2 = Math.pow(x,2);
	   xy = x*y;
	   sum_x  = x  + sum_x;
	   sum_y  = y  + sum_y;
	   sum_x2 = x2 + sum_x2;
	   sum_xy = xy + sum_xy;
	};


	/*TERMԂɂ敨W΍vZ*/
	double sum = 0,average = 0 ,st_div = 0;
	for(i = latestFutureElem - TERM + 1; i <= latestFutureElem  ; i++) sum = sum + futurePrices[i];
	average = sum / TERM;
	sum = 0;
	for(i = latestFutureElem - TERM + 1; i <= latestFutureElem  ; i++) sum = sum + Math.pow(futurePrices[i]-average,2);
	st_div = Math.sqrt(sum / TERM) ;



	double a = 0;
	double b = 0;
	double cc = 0;
	double n0 = 0;

	a    =  ((sum_y * sum_x2) - (sum_x * sum_xy)) / ((TERM * sum_x2) - Math.pow(sum_x,2));
	b    =  ((TERM * sum_xy) - (sum_x * sum_y)) / ((TERM * sum_x2) - Math.pow(sum_x,2));
	cc   =  -1 * (a/b);
	n0   =  futurePrices[(latestFutureElem - TERM)];


	/*ߎߎlZo*/
	double[] app = new double[TERM + 3];
	for (int t = 1 ; t <= TERM + 2 ; t++) app[t] = 1 /( (1 / cc) + ( ( 1 / n0) - (1 / cc))* Math.exp(-1 * a * t));

	/*덷*/
	double err = 0;int t = 0;
	for(i = latestFutureElem - TERM + 1 ,t = 1 ; i <= latestFutureElem; i++)
	    err = err + Math.pow((futurePrices[i] - app[t]),2);

	/*PʕW΍ӂ̌덷 Zo*/
	double st_err = 0;
	st_err = err / st_div;


	/*oۂf*/
	/*
	if (app[TERM + 1] > (latestFuturePrice + 5) ) order.buysell =Order.BUY;
	else if(app[TERM + 1] < (latestFuturePrice - 5) ) order.buysell = Order.SELL;
	else order.buysell = Order.NONE;
	*/

	/*oۂ̔f*/
	/*
	if ((app[TERM + 1] - app[TERM]) > 5 && Math.abs(app[TERM] - latestFuturePrice) < 10) order.buysell =Order.BUY;
	else if(app[TERM + 1] - app[TERM]< - 5 && Math.abs(app[TERM] - latestFuturePrice) < 10)  order.buysell = Order.SELL;
	else order.buysell = Order.NONE;
	*/

	/*oۂ̔f*/
	if ((app[TERM + 1] - app[TERM]) > 8 && app[TERM + 2] - app[TERM + 1] > 3 && st_err < 1000) order.buysell =Order.BUY;
	else if(app[TERM + 1] - app[TERM]< -8 && app[TERM + 2] - app[TERM + 1] < -3 && st_err <1000)  order.buysell = Order.SELL;
	else order.buysell = Order.NONE;


	/*ʁEǐ*/

	order.price = prevPrice + (int) (fWidthOfPrice * fRandom.nextGaussian());
	order.quant = fMinQuant + fRandom.nextInt(fMaxQuant - fMinQuant + 1);
	if( order.buysell != Order.NONE)
	    message("Logistic:i:"+futurePrices[59]+" c:"+restDay+" 덷:"+st_err+"l:"+app[TERM]+","+app[TERM + 1]);

	return order;

    }


    /* (non-Javadoc)
     * @see strategy.UBaseStrategy#setParameters(java.lang.String[])
     *@KvȂI
     */
    public void setParameters(String[] args) {
	super.setParameters(args);
	for (int i = 0; i < args.length; ++i) {
	    StringTokenizer st = new StringTokenizer(args[i], "= ");
	    String key = st.nextToken();
	    String value = st.nextToken();
	    if (key.equals("WidthOfPrice")) {
		fWidthOfPrice = Integer.parseInt(value);
	    } else if (key.equals("MinQuant")) {
		fMinQuant = Integer.parseInt(value);
	    } else if (key.equals("MaxQuant")) {
		fMaxQuant = Integer.parseInt(value);
	    } else if (key.equals("MaxPosition")) {
		fMaxPosition = Integer.parseInt(value);
	    } else if (key.equals("SpreadRatio")) {
		fSpreadRatio = Double.parseDouble(value);
	    } else {
		message("Unknown parameter:" + key
			+ " in RandomStrategy.setParameters");
	    }
	}
    }
}
